import json
import logging
import subprocess
from pathlib import Path
from typing import Tuple, Union

DEVNULL = subprocess.DEVNULL


def getexe(exein: Path=None) -> Tuple[str, str]:
    """checks that host streaming program is installed"""

    if not exein:
        exe = 'ffmpeg'
        probeexe = 'ffprobe'
    else:
        exe = str(Path(exein).expanduser())
        probeexe = str(Path(exein).expanduser().parent / 'ffprobe')
# %% verify
    try:
        subprocess.check_call((exe, '-h'),
                              stdout=DEVNULL, stderr=DEVNULL)
    except FileNotFoundError as e:
        raise FileNotFoundError(f'FFmpeg not found at {exe}  {e}.')

    try:
        subprocess.check_call((probeexe, '-h'),
                              stdout=DEVNULL, stderr=DEVNULL)
    except FileNotFoundError as e:
        raise FileNotFoundError(f'FFprobe: {probeexe}  {e}')

    return exe, probeexe


def get_streams(fn: Path, exe: Path) -> Union[None, dict]:
    if not fn:  # audio-only
        return None

    fn = Path(fn).expanduser()

    assert fn.is_file(), f'{fn} is not a file'

    cmd = [str(exe), '-v', 'error', '-print_format', 'json',
           '-show_streams', '-show_format', str(fn)]

    ret = subprocess.check_output(cmd, universal_newlines=True)
# %% decode JSON from FFprobe
    js = json.loads(ret)
    return js['streams']


def get_resolution(fn: Path, exe: Path) -> Union[None, Tuple[int, int]]:
    """
    get resolution (widthxheight) of video file
    http://trac.ffmpeg.org/wiki/FFprobeTips#WidthxHeight

    FFprobe gets resolution from the first video stream in the file it finds.

    inputs:
    ------
    fn: Path to the video filename

    outputs:
    -------
    res:  (width,height) in pixels of the video

    if not a video file, None is returned.
    """

    streams = get_streams(fn, exe)
    if streams is None:
        return None

    res = None

    for s in streams:
        if s['codec_type'] != 'video':
            continue
        res = (s['width'], s['height'])

    return res


def get_framerate(fn: Path, exe: Path) -> Union[None, float]:
    """
    get framerate of video file
    http://trac.ffmpeg.org/wiki/FFprobeTips#FrameRate

    FFprobe gets framerate from the first video stream in the file it finds.

    inputs:
    ------
    fn: Path to the video filename

    outputs:
    -------
    fps: video framerate (frames/second)

    if not a video file, None is returned.
    """

    streams = get_streams(fn, exe)
    if streams is None:
        return None

    fps = None

    for s in streams:
        if s['codec_type'] != 'video':
            continue
        # https://www.ffmpeg.org/faq.html#toc-AVStream_002er_005fframe_005
        fps = s['avg_frame_rate']
        fps = list(map(int, fps.split('/')))
        try:
            fps = fps[0] / fps[1]
        except ZeroDivisionError:
            logging.error(f'FPS not available:{fn}. Is it a video/audio file?')
            fps = None

    return fps



#!/usr/bin/env python
"""
LIVE STREAM using FFmpeg -- webcam

https://www.scivision.co/youtube-live-ffmpeg-livestream/

Facebook Stream Key: https://www.facebook.com/live/create

Windows: get DirectShow device list from::

   ffmpeg -list_devices true -f dshow -i dummy
"""
import PyLivestream

if __name__ == '__main__':
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    from argparse import ArgumentParser
    p = ArgumentParser(description="livestream microphone audio")
    p.add_argument('site',
                   help='sites to stream [youtube,periscope,facebook,twitch]',
                   nargs='+')
    p.add_argument('-image', help='static image to display.',
                   default='doc/logo.png')
    p.add_argument('-i', '--ini',
                   help='*.ini file with stream parameters',
                   default='stream.ini')
    p.add_argument('-y', '--yes', help='no confirmation dialog',
                   action='store_true')
    P = p.parse_args()

    s = PyLivestream.Microphone(P.ini, P.site, P.image, P.yes)
    sites = list(s.streams.keys())
# %% Go live
    if P.yes:
        print('going live on', sites)
    else:
        input(f"Press Enter to go live on {sites}.    Or Ctrl C to abort.")

    s.golive()



from pathlib import Path
import subprocess as sp
from typing import List, Union, Dict
#
from . import stream


class Livestream(stream.Stream):

    def __init__(self, ini: Path, site: str, vidsource: str=None,
                 image: Path=None, loop: bool=False, infn: Path=None,
                 yes: bool=False) -> None:
        super().__init__(ini, site, vidsource, image, loop, infn, yes=yes)

        self.site = site.lower()

        self.osparam()

        self.video_bitrate()

        vid1, vid2 = self.videostream()

        aud1: List[str] = self.audiostream()
        aud2: List[str] = self.audiocomp()

        buf: List[str] = self.buffer(self.server)
# %% begin to setup command line
        cmd: List[str] = [str(self.exe)]

        if self.yes:
            cmd += ['-y']

        cmd += vid1 + aud1 + vid2 + aud2 + buf

        streamid: str = self.key if self.key else ''

        self.sink: List[str] = [self.server + streamid]

        self.cmd: List[str] = cmd + self.sink

    def golive(self, sinks: List[str]=None):
        """finally start the stream(s)"""

        if self.key is None and self.server not in ('NUL', '/dev/null'):
            print('\n', ' '.join(self.cmd), '\n')
            return

        if not sinks:  # single stream
            print('\n', ' '.join(self.cmd))
            sp.check_call(self.cmd)
        else:  # multi-stream output tee
            cmdstem = self.cmd[:-3]
            # +global_header is necessary to tee to multiple services
            cmd = cmdstem + ['-flags:v', '+global_header',
                             '-f', 'tee']

            if self.image:
                #  connect image to video stream, audio file to audio stream
                cmd += ['-map', '0:v', '-map', '1:a']
            else:
                if self.vidsource == 'file':
                    # picks first video and audio stream, often correct
                    cmd += ['-map', '0:v', '-map', '0:a:0']
                else:  # device (webcam)
                    # connect video device to video stream,
                    # audio device to audio stream
                    cmd += ['-map', '0:v', '-map', '1:a']

            cmd += ['[f=flv]' + '|[f=flv]'.join(sinks)]  # no double quotes
            print(' '.join(cmd))

            sp.check_call(cmd)


# %% operators
class Screenshare(Livestream):

    def __init__(self, ini: Path, websites: Union[str, List[str]],
                 yes: bool=False) -> None:

        vidsource = 'screen'

        if isinstance(websites, str):
            websites = [websites]

        streams = {}
        for site in websites:
            streams[site] = Livestream(ini, site, vidsource, yes=yes)

        self.streams: Dict[str, Livestream] = streams

    def golive(self):

        sinks: List[str] = [self.streams[stream].sink[0]
                            for stream in self.streams]

        self.streams[stream.unify_streams(self.streams)].golive(sinks)


class Webcam(Livestream):

    def __init__(self, ini: Path, websites: Union[str, List[str]],
                 yes: bool=False) -> None:

        vidsource = 'camera'

        if isinstance(websites, str):
            websites = [websites]

        streams = {}
        for site in websites:
            streams[site] = Livestream(ini, site, vidsource, yes=yes)

        self.streams: Dict[str, Livestream] = streams

    def golive(self):

        sinks: List[str] = [self.streams[stream].sink[0]
                            for stream in self.streams]

        self.streams[stream.unify_streams(self.streams)].golive(sinks)


class Microphone(Livestream):

    def __init__(self, ini: Path, sites: Union[str, List[str]], image: Path,
                 yes: bool=False) -> None:

        if isinstance(sites, str):
            sites = [sites]

        streams = {}
        for site in sites:
            streams[site] = Livestream(ini, site, image=image, yes=yes)

        self.streams: Dict[str, Livestream] = streams

    def golive(self):

        sinks: List[str] = [self.streams[stream].sink[0]
                            for stream in self.streams]

        self.streams[stream.unify_streams(self.streams)].golive(sinks)


# %% File-based inputs
class FileIn(Livestream):

    def __init__(self, ini: Path, sites: Union[str, List[str]], infn: Path,
                 loop: bool=False, image: Path=None, yes: bool=False) -> None:

        vidsource = 'file'

        if isinstance(sites, str):
            sites = [sites]

        streams = {}
        for site in sites:
            streams[site] = Livestream(ini, site, vidsource, image=image,
                                       loop=loop, infn=infn, yes=yes)

        self.streams: Dict[str, Livestream] = streams

    def golive(self):

        sinks: List[str] = [self.streams[stream].sink[0]
                            for stream in self.streams]

        self.streams[stream.unify_streams(self.streams)].golive(sinks)


class SaveDisk(stream.Stream):

    def __init__(self, ini: Path, outfn: Path=None,
                 clobber: bool=False) -> None:
        """
        records to disk screen capture with audio

        if not outfn, just cite command that would have run
        """
        site = 'file'
        vidsource = 'screen'

        super().__init__(ini, site, vidsource)

        self.outfn = Path(outfn).expanduser() if outfn else None

        self.osparam()

        vid1, vid2 = self.videostream()

        aud1 = self.audiostream()
        aud2 = self.audiocomp()

        self.cmd: List[str] = [str(self.exe)] + vid1 + aud1 + vid2 + aud2

        # ffmpeg relies on suffix for container type, this is a fallback.
        if self.outfn and not self.outfn.suffix:
            self.cmd += ['-f', 'flv']

        self.cmd += [str(self.outfn)]

        if self.timelimit:
            self.cmd += ['-timelimit', self.timelimit]

        if clobber:
            self.cmd += ['-y']

#        if sys.platform == 'win32':
#            cmd += ['-copy_ts']

    def save(self):

        print('\n', ' '.join(self.cmd), '\n')

        if self.outfn:
            try:
                ret = sp.check_call(self.cmd)
                print('FFmpeg returncode', ret)
            except FileNotFoundError:
                pass
        else:
            print('specify filename to save screen capture w/ audio to disk.')



import bisect
from pathlib import Path
import logging
import os
import sys
from configparser import ConfigParser
from typing import Tuple, List, Dict
#
from . import sio

# %%  Col0: vertical pixels (height). Col1: video kbps. Interpolates.
# NOTE: Python >= 3.6 has guaranteed dict() order.

BR30 = dict([
        (240, 300),
        (360, 400),
        (480, 500),
        (540, 800),
        (720, 1800),
        (1080, 3000),
        (1440, 6000),
        (2160, 13000)])

BR60 = dict([
       (720, 2250),
       (1080, 4500),
       (1440, 9000),
       (2160, 20000)])

FPS = 30.  # default frames/sec if not defined otherwise


# %% top level
class Stream:

    def __init__(self, ini: Path, site: str, vidsource: str=None,
                 image: Path=None, loop: bool=False, infn: Path=None,
                 yes: bool=False) -> None:

        self.ini: Path = Path(ini).expanduser()
        self.site: str = site
        self.vidsource = vidsource
        self.image = Path(image).expanduser() if image else None
        self.loop = loop

        self.infn = Path(infn).expanduser() if infn else None
        self.yes = yes

    def osparam(self):
        """load OS specific config"""

        assert self.ini.is_file(), f'{self.ini} not found.'

        C = ConfigParser(inline_comment_prefixes=('#', ';'))
        C.read(str(self.ini))

        assert self.site in C, f'{self.site} not found: {self.ini}'

        if 'XDG_SESSION_TYPE' in os.environ:
            if os.environ['XDG_SESSION_TYPE'] == 'wayland':
                logging.error('Wayland may only give black output. Try X11')

        self.exe, self.probeexe = sio.getexe(C.get(sys.platform, 'exe',
                                                   fallback='ffmpeg'))

        if self.vidsource == 'camera':
            self.res: Tuple[int, int] = C.get(self.site,
                                              'webcam_res').split('x')
            self.fps: float = C.getint(self.site, 'webcam_fps')
        elif self.vidsource == 'screen':
            self.res: Tuple[int, int] = C.get(self.site,
                                              'screencap_res').split('x')
            self.fps: float = C.getint(self.site, 'screencap_fps')
            self.origin: Tuple[int, int] = C.get(self.site,
                                                 'screencap_origin').split(',')
        elif self.vidsource is None or self.vidsource == 'file':
            self.res: Tuple[int, int] = sio.get_resolution(self.infn,
                                                           self.probeexe)
            self.fps: float = sio.get_framerate(self.infn, self.probeexe)
        else:
            raise ValueError(f'unknown video source {self.vidsource}')

        self.audiofs: int = C.get(self.site, 'audiofs')  # not getint
        self.preset: str = C.get(self.site, 'preset')
        self.timelimit: str = C.get(self.site, 'timelimit', fallback=None)

        self.videochan: str = C.get(sys.platform, 'videochan')
        self.audiochan: str = C.get(sys.platform, 'audiochan', fallback=None)
        self.vcap: str = C.get(sys.platform, 'vcap')
        self.acap: str = C.get(sys.platform, 'acap', fallback=None)
        self.hcam: str = C.get(sys.platform, 'hcam')

        self.video_kbps: int = C.getint(self.site, 'video_kbps', fallback=None)
        self.audio_bps: int = C.get(self.site, 'audio_bps')

        self.keyframe_sec: int = C.getint(self.site, 'keyframe_sec')

        self.server: str = C.get(self.site, 'server', fallback=None)
# %% Key (hexaecimal stream ID)
        keyfn = C.get(self.site, 'key', fallback=None)
        if not keyfn:  # '' or None
            self.key = None
        else:
            # .resolve()  # Python >= 3.6 required for .resolve(strict=False)
            key = Path(keyfn).expanduser()
            self.key = key.read_text().strip() if key.is_file() else keyfn

    def videostream(self) -> Tuple[List[str], List[str]]:
        """optimizes video settings"""
# %% configure video input
        if self.vidsource == 'screen':
            vid1 = self.screengrab()
        elif self.vidsource == 'camera':
            vid1 = self.webcam()
        elif self.vidsource is None or self.vidsource == 'file':
            vid1 = self.filein()
        else:
            raise ValueError(f'unknown vidsource {self.vidsource}')
# %% configure video output
        vid2: List[str] = ['-c:v', 'libx264', '-pix_fmt', 'yuv420p']
# %% set frames/sec, bitrate and keyframe interval
        """
         DON'T DO THIS.
         It makes keyframes/bitrate far off from what streaming sites want
         vid2 += ['-tune', 'stillimage']

         The settings below still save video/data bandwidth for the still image
         + audio case.
        """
        fps = self.fps if self.fps is not None else FPS

        vid2 += ['-preset', self.preset,
                 '-b:v', str(self.video_kbps) + 'k']

        if self.image:
            vid2 += ['-r', str(fps)]

        vid2 += ['-g', str(self.keyframe_sec * fps)]

        return vid1, vid2

    def audiostream(self) -> List[str]:
        """
        -ac * may not be needed, took out.
        -ac 2 NOT -ac 1 to avoid "non monotonous DTS in output stream" errors
        """
        if not self.audio_bps or not self.acap or not self.audiochan:
            return []

        if not self.vidsource == 'file':
            return ['-f', self.acap, '-i', self.audiochan]
        else:  # file input
            return []
#        else: #  file input
#            return ['-ac','2']

    def audiocomp(self) -> List[str]:
        """select audio codec
        https://trac.ffmpeg.org/wiki/Encode/AAC#FAQ
        https://support.google.com/youtube/answer/2853702?hl=en
        https://www.facebook.com/facebookmedia/get-started/live
        """

        if not self.audio_bps or not self.acap or not self.audiochan:
            return []

        return ['-c:a', 'aac',
                '-b:a', str(self.audio_bps),
                '-ar', str(self.audiofs)]

    def video_bitrate(self):
        """get "best" video bitrate.
        Based on YouTube Live minimum specified stream rate."""
        if self.video_kbps:  # per-site override
            return

        if self.res is not None:
            x: int = int(self.res[1])
        elif self.vidsource is None or self.vidsource == 'file':
            logging.info('assuming 480p input.')
            x = 480
        else:
            raise ValueError('Unsure of your video resolution request.'
                             'Try setting video_kpbs in the .ini file.')

        if self.fps is None or self.fps <= 30:
            self.video_kbps: int = list(BR30.values())[
                                    bisect.bisect_left(list(BR30.keys()), x)]
        else:
            self.video_kbps: int = list(BR60.values())[
                                    bisect.bisect_left(list(BR60.keys()), x)]

    def screengrab(self) -> List[str]:
        """choose to grab video from desktop. May not work for Wayland."""
        vid1 = ['-f', self.vcap,
                '-r', str(self.fps)]

        if self.res is not None:
            vid1 += ['-s', 'x'.join(map(str, self.res))]

        if sys.platform == 'linux':
            vid1 += [f'-i', ':0.0+{self.origin[0]},{self.origin[1]}']
        elif sys.platform == 'win32':
            vid1 += ['-offset_x', self.origin[0], '-offset_y', self.origin[1],
                     '-i', self.videochan]
        elif sys.platform == 'darwin':
            vid1 += ['-i', "0:0"]

        return vid1

    def webcam(self) -> List[str]:
        """configure webcam"""
        vid1 = ['-f', self.hcam,
                '-r', str(self.fps),
                '-i', self.videochan]

        return vid1

    def filein(self) -> List[str]:
        """stream input file  (video, or audio + image)"""

        vid1: List[str] = []

        """
        -re is NOT for actual streaming devices (webcam, microphone)
        https://ffmpeg.org/ffmpeg.html
        """

        if self.image:
            vid1 += ['-loop', '1', '-f', 'image2']

        if self.image or self.vidsource == 'file':
            vid1 += ['-re']

        if self.loop:
            vid1 += ['-stream_loop', '-1']  # FFmpeg >= 3

        if self.image:  # still image, for audio-only input files
            vid1 += ['-i', str(self.image)]

        if self.infn:
            vid1 += ['-i', str(self.infn)]

        return vid1

    def buffer(self, server: str) -> List[str]:
        """configure network buffer. Tradeoff: latency vs. robustness"""
        # constrain to single thread, default is multi-thread
        # buf = ['-threads', '1']

        buf = ['-maxrate', f'{self.video_kbps}k',
               '-bufsize', f'{self.video_kbps//2}k']

        if self.image:  # static image + audio
            buf += ['-shortest']

        # for very old versions of FFmpeg, such as Ubuntu 16.04
        # still OK for current FFmpeg versions too
        buf += ['-strict', 'experimental']

        # must manually specify container format when streaming to web.
        buf += ['-f', 'flv']

        return buf


def unify_streams(streams: Dict[str, Stream]) -> str:
    """
    find least common denominator stream settings,
        so "tee" output can generate multiple streams.
    First try: use stream with lowest video bitrate.

    Exploits that Python >= 3.6 has guaranteed dict() ordering.

    fast native Python argmin()
    https://stackoverflow.com/a/11825864
    """
    vid_bw: List[int] = [streams[stream].video_kbps for stream in streams]

    argmin: int = min(range(len(vid_bw)), key=vid_bw.__getitem__)

    key: str = list(streams.keys())[argmin]

    return key



#!/usr/bin/env python
"""
Save to disk screen capture for upload to YOUTUBE using FFMPEG

https://www.scivision.co/youtube-ffmpeg-screen-capture-with-audio/
https://trac.ffmpeg.org/wiki/Capture/Desktop
https://support.google.com/youtube/answer/2853702

Windows: get DirectShow device list from:
   ffmpeg -list_devices true -f dshow -i dummy
"""
import PyLivestream

if __name__ == '__main__':
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    from argparse import ArgumentParser
    p = ArgumentParser()
    p.add_argument('outfn', help='video file to save to disk.')
    p.add_argument('-c', '--clobber', help='overwrite file with same name',
                   action='store_true')
    p.add_argument('-i', '--ini', help='*.ini file with stream parameters',
                   default='stream.ini')
    p.add_argument('-y', '--yes', help='no confirmation dialog',
                   action='store_true')
    P = p.parse_args()

    s = PyLivestream.SaveDisk(P.ini, P.outfn, P.clobber)
# %%
    if P.yes:
        print('saving screen capture to', s.outfn)
    else:
        input(f"Press Enter to screen capture to file {s.outfn}."
              "Or Ctrl C to abort.")

    s.save()



#!/usr/bin/env python
"""
LIVE STREAM using FFMPEG -- Globbed input file list

NOTE: most video streaming sites REQUIRE you have to a video feed,
     so try the `-image` option with a favorite .JPG or whatever.

https://www.scivision.co/youtube-live-ffmpeg-livestream/
https://support.google.com/youtube/answer/2853702
"""
from pathlib import Path
from typing import List
import PyLivestream


def playonce(flist: List[Path], image: Path, site: str, inifn: Path,
             yes: bool):

    for f in flist:
        s = PyLivestream.FileIn(inifn, site, infn=f, loop=False, image=image,
                                yes=yes)

        s.golive()


if __name__ == '__main__':
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    from argparse import ArgumentParser
    p = ArgumentParser(description="Livestream a globbed input file list")
    p.add_argument('site',
                   help='site to stream: [youtube,periscope,facebook,twitch]',
                   nargs='+')
    p.add_argument('path', help='path to discover files from')
    p.add_argument('glob', help='file glob pattern to stream.')
    p.add_argument('-i', '--ini', help='*.ini file with stream parameters',
                   default='stream.ini')
    p.add_argument('-image',
                   help='static image to display, for audio-only files.')
    p.add_argument('-loop', help='repeat the globbed file list endlessly',
                   action='store_true')
    p.add_argument('-y', '--yes', help='no confirmation dialog',
                   action='store_true')
    P = p.parse_args()

    path = Path(P.path).expanduser()
    flist = list(path.glob(P.glob))

    if not flist:
        raise FileNotFoundError(f'did not find files with {path} {P.glob}')

    print('streaming these files. Be sure list is correct! \n')
    print('\n'.join(map(str, flist)))
    print()

    if P.yes:
        print('going live on', P.site)
    else:
        input(f"Press Enter to go live on {P.site}.    Or Ctrl C to abort.")

    inifn = Path(P.ini).expanduser()

    if P.loop:
        while True:
            playonce(flist, P.image, P.site, inifn, P.yes)
    else:
        playonce(flist, P.image, P.site, inifn, P.yes)



#!/usr/bin/env python
"""
LIVE STREAM using FFMPEG -- Looping input file endlessly

NOTE: for audio files,
     use FileGlob2Livestream.py with options `-image myimg.jpg -loop`

https://www.scivision.co/youtube-live-ffmpeg-livestream/
https://support.google.com/youtube/answer/2853702
"""
from typing import List
import PyLivestream

if __name__ == '__main__':
    import signal
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    from argparse import ArgumentParser
    p = ArgumentParser(description="Livestream a single looped input file")
    p.add_argument('infn', help='file to stream, looping endlessly.')
    p.add_argument('site',
                   help='site to stream: [youtube,periscope,facebook,twitch]',
                   nargs='+')
    p.add_argument('-i', '--ini', help='*.ini file with stream parameters',
                   default='stream.ini')
    p.add_argument('-y', '--yes', help='no confirmation dialog',
                   action='store_true')
    P = p.parse_args()

    S = PyLivestream.FileIn(P.ini, P.site, infn=P.infn,
                            loop=True, yes=P.yes)
    sites: List[str] = list(S.streams.keys())
# %% Go live
    if P.yes:
        print(f'going live on {sites} looping file {P.infn}')
    else:
        input(f"Press Enter to go live on {sites},"
              f"looping file {P.infn}")
        print('Or Ctrl C to abort.')

    S.golive()



#!/usr/bin/env python
from typing import List
from setuptools import setup, find_packages

install_requires: List[str] = []
tests_require: List[str] = ['pytest', 'nose', 'coveralls', 'flake8', 'mypy']


setup(name='PyLivestream',
      packages=find_packages(),
      version='1.6.5',
      author='Michael Hirsch, Ph.D.',
      url='https://github.com/scivision/PyLivestream',
      description=('Easy streaming using FFmpeg to '
                   'YouTube Live, Periscope, Facebook Live, Twitch, ...'),
      long_description=open("README.rst").read(),
      python_requires='>=3.6',
      install_requires=install_requires,
      tests_require=tests_require,
      extras_require={'tests': tests_require},
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Console',
          'Intended Audience :: End Users/Desktop',
          'Operating System :: OS Independent',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Topic :: Multimedia :: Graphics :: Capture :: Screen Capture',
          'Topic :: Multimedia :: Sound/Audio :: Capture/Recording',
          'Topic :: Multimedia :: Video :: Capture',
      ],
      scripts=['FileGlobLivestream.py', 'ScreenshareLivestream.py',
               'FileLoopLivestream.py', 'ScreenCapture2disk.py',
               'WebcamLivestream.py', 'MicrophoneLivestream.py'],
      include_package_data=True,
      )
